import logging

from collections import namedtuple
from typing import List, Union, Dict, Optional, Any

from .api import event, task_event
from .appliance import min_ppa_version

logger = logging.getLogger(__name__)

TableTabs = Dict[str, List[list]]
TableRows = List[list]
TableData = Union[TableTabs, TableRows]


def _validate_color(color: Optional[str]):
    if color is None:
        return color
    try:
        # Remove leading hash before testing the hex code
        int(color[1:] if color.startswith("#") else color, 16)
        return color if color.startswith("#") else f"#{color}"
    except ValueError:
        raise ValueError(f"Color value {color} is not valid hexadecimal.")


class FileUpload(namedtuple("FileUpload", ("data", "filename", "size", "mtime", "type"))):
    def __repr__(self):
        return f"filename '{self.filename}', size '{self.size}', last_modified '{self.mtime}', type '{self.type}'"


@min_ppa_version("2.11.0")
@task_event()
def input_form(text: str, fields: List[Dict], element_id: Optional[str] = None, sensitive: Optional[bool] = None):
    return event("input/form", text, fields=fields, id=element_id, obfuscate=sensitive)


@task_event(return_tuple=FileUpload)
def input_file(text: str, accept: Union[str, List[str]] = None, element_id: Optional[str] = None) -> FileUpload:
    return event("input/file", text, accept=accept, id=element_id)


@min_ppa_version("2.6.3")
@task_event(return_tuple=FileUpload)
def input_sensitive_file(
    text: str, accept: Union[str, List[str]] = None, element_id: Optional[str] = None, sensitive: Optional[bool] = None
) -> FileUpload:
    return event("input/sensitive_file", text, accept=accept, id=element_id, obfuscate=sensitive)


@task_event()
def input_choice(
    text: str,
    *,
    options: List[Union[str, int]],
    default: Optional[Union[str, int]] = None,
    element_id: Optional[str] = None,
    sensitive: Optional[bool] = None,
) -> str:
    return event("input/choice", text, options=options, default_option=default, id=element_id, obfuscate=sensitive)


@min_ppa_version("2.10.0")
@task_event()
def input_checklist(
    text: str,
    *,
    options: List[Union[str, int]],
    defaults: Optional[List[Union[str, int]]] = None,
    element_id: Optional[str] = None,
    sensitive: Optional[bool] = None,
) -> str:
    return event(
        "input/checklist", text, options=list(options), default_options=defaults, id=element_id, obfuscate=sensitive
    )


@task_event()
def input_table(
    text: str,
    *,
    header: List[str],
    data: TableRows,
    element_id: Optional[str] = None,
    sensitive: Optional[bool] = None,
) -> List[int]:
    return event(
        "input/table", text, data=data, type="application/json", header=header, id=element_id, obfuscate=sensitive
    )


@task_event()
def input_accept(text: str, element_id: Optional[str] = None) -> None:
    return event("input/accept", text, id=element_id)


@task_event()
def input_confirm(
    text: str,
    confirm: Optional[str] = None,
    cancel: Optional[str] = None,
    element_id: Optional[str] = None,
    sensitive: Optional[bool] = None,
) -> bool:
    return event("input/confirm", text, id=element_id, confirm=confirm, cancel=cancel, obfuscate=sensitive)


@task_event()
def input_password(text: str, meter: bool = False, element_id: Optional[str] = None) -> str:
    return event("input/password", text, id=element_id, meter=meter)


@task_event()
def input_text(
    text: str,
    default: Optional[str] = None,
    pattern: str = ".*",
    pattern_error: str = "",
    element_id: Optional[str] = None,
    required: bool = False,
    sensitive: Optional[bool] = None,
) -> str:
    return event(
        "input/text",
        text,
        default=default,
        pattern=pattern,
        required=required,
        id=element_id,
        pattern_error=pattern_error,
        obfuscate=sensitive,
    )


@min_ppa_version("2.6.3")
@task_event()
def input_text_area(
    text: str,
    default: Optional[str] = None,
    pattern: str = ".*",
    pattern_error: str = "",
    element_id: Optional[str] = None,
    required: bool = False,
    rows: int = 5,
    max_rows: int = 50,
    sensitive: Optional[bool] = None,
) -> str:
    return event(
        "input/text",
        text,
        default=default,
        pattern=pattern,
        required=required,
        id=element_id,
        multiline=True,
        rows=rows,
        maxRows=max_rows,
        obfuscate=sensitive,
        patternError=pattern_error,
    )


@min_ppa_version("2.8.0")
@task_event()
def input_timestamp(text: str) -> str:
    return event("input/timestamp", text)


@min_ppa_version("2.11.0")
@task_event()
def input_date(text: str) -> Dict[str, Any]:
    return event("input/date", text)


@task_event()
def output_markdown(doc: str, sensitive: Optional[bool] = None, element_id: Optional[str] = None) -> Union[Dict, List]:
    return event("output/markdown", doc, id=element_id, obfuscate=sensitive)


@task_event()
def output_error(
    text: str, color: Optional[str] = None, element_id: Optional[str] = None, sensitive: Optional[bool] = None
) -> None:
    return event("output/error", text, id=element_id, color=_validate_color(color), obfuscate=sensitive)


@task_event()
def output_info(
    text: str, color: Optional[str] = None, sensitive: Optional[bool] = None, element_id: Optional[str] = None
) -> None:
    return event("output/info", text, id=element_id, color=_validate_color(color), obfuscate=sensitive)


@task_event()
def output_file(
    text: str, *, data: str, filename: str, type: str = "application/octet-stream", element_id: Optional[str] = None
) -> None:
    return event("output/file", text, filename=filename, data=data, type=type, id=element_id)


@task_event()
def output_progress(text: str, *, percent: Union[int, None], element_id: Optional[str] = None) -> None:
    return event("output/progress", text, percent=percent, id=element_id)


@task_event()
def output_table(
    text: str,
    *,
    header: List[str],
    data: TableData,
    element_id: Optional[str] = None,
    sensitive: Optional[bool] = None,
) -> None:
    return event(
        "output/table", text, data=data, type="application/json", header=header, id=element_id, obfuscate=sensitive
    )
