import logging

from typing import Callable

from ppa import ui, appliance, exceptions


logger = logging.getLogger(__name__)


# The min_ppa_version decorator in SDK doesn't display the error, this wrapped version does.
def min_ppa_version(minimum_version: str) -> Callable:
    def _wrapped_version_check(func):
        def _execute(*args, **kwargs):
            try:
                return appliance.min_ppa_version(minimum_version)(func)(*args, **kwargs)
            except exceptions.PPAVersionError as e:
                ui.output_error(str(e))
                raise e

        return _execute

    return _wrapped_version_check


def argument_minimum_versions(**versioned_args):
    def _wrapper(func):
        def _execute(*args, **kwargs):
            ignored_kwargs = []
            for argument, minimum_version in versioned_args.items():
                # If the keyword argument is not None (None is the default value - if it's not
                # None, it was supplied by the playbook), & the PPA version is less than the
                # minimum, display an error & ignore the keyword argument.
                if all((kwargs.get(argument) is not None, appliance.current_version_less_than(minimum_version))):
                    message = f"The '{argument}' input supplied to {func.__name__} will be ignored as it requires PPA version {minimum_version} or newer."
                    ui.output_error(message)
                    logger.error(message)
                    ignored_kwargs.append(argument)
            included_kwargs = {k: v for k, v in kwargs.items() if k not in ignored_kwargs}
            return func(*args, **included_kwargs)

        return _execute

    return _wrapper
