"""
User

This module contains actions related to the user running a task.
"""
import os
import uuid
import logging

from typing import Optional

from ppa import user, ui, exceptions

from ._common import min_ppa_version


logger = logging.getLogger(__name__)


@min_ppa_version("3.0.0")
def has_mfa(text: Optional[str] = "Getting MFA Configuration") -> bool:
    """Find out whether the user who started the task has MFA configured in the PPA Appliance.

    !!! question "Minimum Plugin Version: 8.0.0"

    !!! question "Minimum PPA Version: 3.0.0"

    ??? info "Input"

        - __text:__ optional text to display when the action runs (defaults to _Getting MFA Configuration_)

    ??? success "Output"

        A boolean is outputted by this action:

        - `true` if the user has MFA configured

        - `false` if the user does not have MFA configured

    ??? example
        ```YAML linenums="1"
        - ppa.user.has_mfa:
          save: has_mfa
        ```"""
    return user.has_mfa(text)


@min_ppa_version("3.0.0")
def prompt_mfa(
    text: Optional[str] = "Please provide an MFA token.", max_attempts: Optional[int] = 5, sensitive: Optional[bool] = None
) -> None:
    """Prompt the user who started the task for MFA.

    This action will fail if the user does not have MFA configured in the PPA appliance.

    !!! question "Minimum Plugin Version: 8.0.0"

    !!! question "Minimum PPA Version: 3.0.0"

    ??? info "Input"

        - __text:__ optional text to display as the MFA prompt (defaults to _Please provide an MFA token._)

        - __max_attempts:__ maximum number of authentication attempts before the action fails (defaults to _5_)

        - __sensitive:__ set to `true` to [protect sensitive data](../sensitive_data) (requires __PPA v3__ & __plugin version 8.0.0__)

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example

        ```YAML linenums="1"
        - ppa.user.prompt_mfa:
        ```"""

    ppa_user = os.environ.get("PPA_USER")
    try:
        _, username = ppa_user.split("\\")
    except ValueError:
        username = ppa_user

    if has_mfa() is False:
        message = f"MFA is not configured for {username}."
        ui.output_error(message)
        raise exceptions.EventConfigurationError(message)

    input_element_id, error_element_id = list(map(lambda _: str(uuid.uuid4()), [0, 1]))
    for attempt in range(1, (max_attempts + 1)):
        remaining_attempts = max_attempts - attempt
        try:
            user.prompt_mfa(text, element_id=input_element_id, sensitive=sensitive)
            return None
        except exceptions.AuthenticationFailed:
            if attempt < max_attempts:
                ui.output_error(
                    f"MFA token rejected, please try again ({remaining_attempts} attempts left).", element_id=error_element_id
                )
                continue
    else:
        message = f"Cancelling MFA authentication after {max_attempts} failed attempts."
        ui.output_error(message)
        raise exceptions.AuthenticationFailed(message)
